@@ -15,6 +15,7 @@ import numpy as np
 import visual_graph_datasets.typing as tc
 from visual_graph_datasets.util import NULL_LOGGER
 from visual_graph_datasets.util import merge_optional_lists
+from visual_graph_datasets.util import merge_nested_dicts
 
 
 def str_presenter(dumper, data):
@@ -128,24 +129,60 @@ def load_visual_graph_dataset_metadata(path: str,
                                   if name.startswith('.')}
 
     data = {}
-    # There is one special file name which is the most important: ".meta.yaml". This file contains the bulk
+    # There is one special file name which is the most important: ".meta.yml". This file contains the bulk
     # of the metadata encoded as a yaml file, so we load that file here, if it exists
-    if '.meta.yaml' in file_map:
+    if '.meta.yml' in file_map:
         logger.info('loading dataset metadata from .meta.yaml file...')
-        file_path = file_map['.meta.yaml']
+        file_path = file_map['.meta.yml']
         with open(file_path, mode='r') as file:
             data.update(yaml.load(file, Loader=yaml.FullLoader))
     else:
         logger.info('no .meta.yaml file found for the dataset!')
 
+    # 21.03.2023 - As a convenience feature we will add the absolute path towards the pre-processing module
+    # process.py to the metadata as well.
+    data['process_path'] = os.path.join(path, 'process.py')
+
     # TODO: Add support for arbitrary metadata files
     return data, {}
 
 
+def load_visual_graph_element(path: str,
+                              name: str,
+                              ) -> dict:
+    """
+    A utility function to load a single visual graph element given the folder ``path`` for the folder in
+    which its files are stored in and the ``name`` of the element. Since a visual graph element consists
+    of multiple files, all files are expected to have the same name!.
+
+    :param path: The string absolute path to the FOLDER which contains the element files
+    :param name: The name, which all the element files share (without any file extensions!)
+
+    :returns: A dictionary which represents the element. It has two fields 'metadata' which contains the
+        full metadata dictionary of the element, and 'image_path' which contains the absolute path to the
+        corresponding visualization image.
+    """
+    element_data = {}
+
+    metadata_path = os.path.join(path, f'{name}.json')
+    with open(metadata_path, file='r') as file:
+        content = file.read()
+        metadata = json.loads(content)
+
+    element_data['metadata'] = metadata
+
+    image_path = os.path.join(path, f'{name}.png')
+    element_data['image_path'] = image_path
+
+    return element_data
+
+
+
 def load_visual_graph_dataset(path: str,
                               logger: logging.Logger = NULL_LOGGER,
                               log_step: int = 100,
                               metadata_contains_index: bool = False,
+                              subset: t.Optional[int] = None,
                               image_extensions: t.Optional[t.List[str]] = ['png'],
                               text_extensions: t.Optional[t.List[str]] = None,
                               metadata_extensions: t.List[str] = ['json'],
@@ -250,6 +287,9 @@ def load_visual_graph_dataset(path: str,
 
                 dataset_index_map[index] = dataset_name_map[name]
 
+            if subset is not None and c > subset:
+                break
+
             if c % log_step == 0:
                 elapsed_time = time.time() - start_time
                 remaining_time = (num_files - c) * (elapsed_time / (c + 1))
@@ -261,6 +301,59 @@ def load_visual_graph_dataset(path: str,
     return metadata_map, dataset_index_map
 
 
+def load_visual_graph_dataset_expansion(index_data_map: tc.VisGraphIndexDict,
+                                        expansion_path: str,
+                                        logger: logging.Logger = NULL_LOGGER,
+                                        log_step: int = 100):
+
+    assert os.path.exists(expansion_path), (
+        f'The given visual graph dataset expansion path {expansion_path} does not exist and can thus '
+        f'not be loaded! Please make sure the path exists and is spelled correctly.'
+    )
+
+    assert os.path.isdir(expansion_path), (
+        f'The given visual graph dataset expansion path {expansion_path} is not a directory! A VGD '
+        f'expansion has to be a directory much like a VGD dataset itself aswell.'
+    )
+
+    # We open the folder, iterate all the files and check if the described elements correlate with any of
+    # the elements in the given visual graph dataset.
+    base_path = expansion_path
+    file_names = os.listdir(expansion_path)
+    num_files = len(file_names)
+
+    for c, file_name in enumerate(file_names):
+        # As with the visual graph datasets themselves we also ignore files starting with a period
+        if file_name.startswith('.'):
+            continue
+
+        # Technically at this point I think it makes sense to only accept json metadata files I dont see
+        # what the use of other file types would be right now
+        if file_name.endswith('.json'):
+
+            file_path = os.path.join(base_path, file_name)
+            with open(file_path) as file:
+                content = file.read()
+                metadata = json.loads(content)
+
+            # Now we need to check if this data matches any of the existing elements in the given VGD
+            # At this point we do this with the "index" information but in the future we can think about
+            # if we need to support a different method as well
+            index = metadata['index']
+            if index in index_data_map:
+                metadata_original = index_data_map[index]['metadata']
+                # The two metadata dicts now need to be nested merged and then that version should be used
+                # in the VGD
+                metadata_merged = merge_nested_dicts(metadata_original, metadata)
+                index_data_map[index]['metadata'] = metadata_merged
+
+            else:
+                logger.info(f' ! not found original entry for index {index}')
+
+        if c % log_step == 0:
+            logger.info(f' * ({c}/{num_files}) processed')
+
+
 # This function is not to be confused with the functions above "generate_visual_graph_dataset_metadata"
 # and "load_visual_graph_dataset_metadata". These two are rather simple ones which have singular and well
 # defined purpose. This function is more complex: It will load a visual graph dataset given by its path,
