@@ -46,13 +46,14 @@ class GraphGenerator:
         # edge under the assumption we make that in the graph growing process every node contributes one
         # edge to the node
         self.seed_node_count = sum(len(g['node_indices']) for g in seed_graphs)
-        if seed_graphs:
+        if seed_graphs and self.seed_node_count > 0:
             self.seed_edge_count = sum(len(g['edge_indices']) for g in seed_graphs)
         else:
             if self.is_directed:
                 self.seed_edge_count = -1
             else:
                 self.seed_edge_count = -2
+
         # undirected graphs will be assumed to just be directed graphs where nodes are connected by two
         # edges pointing both ways.
         if self.is_directed:
@@ -148,7 +149,7 @@ class GraphGenerator:
         # The whole generation procedure is a seeding method. The graph starts out with a seed center and
         # the rest of the graph is then "grown" around that. It is possible to define subgraphs which act
         # as seeds. If that is not the case, then one node is chosen at random to be the seed
-        if len(self.seed_graphs) == 0:
+        if len(self.seed_graphs) == 0 or self.seed_node_count == 0:
             self.insert_seed_node()
         else:
             self.insert_seed_graphs()
@@ -266,21 +267,36 @@ class GraphGenerator:
         # random nodes.
         additional_edge_count = self.num_additional_edges
 
-        # Obviously we only need to connect up the diconnected seeds IF there is more than one seed
+        # Obviously we only need to connect up the disconnected seeds IF there is more than one seed
         if len(self.seed_graphs) > 1:
-            for graph_index in range(len(self.seed_graphs) - 1):
-                if (len(self.seed_graphs[graph_index]['node_indices']) == 0 or
-                        len(self.seed_graphs[graph_index + 1]['node_indices']) == 0):
+            # We essentially want to look at pairs of seed graphs here and connect up the corresponding
+            # still disconnected graphs. These two are the running seed graph indices for the two elements
+            # of that pairwise consideration.
+            gi = 0
+            gj = 1
+            while gj < len(self.seed_graphs):
+                # Because there is the real possibility of completely empty seed graphs, this first section
+                # of the loop is dedicated to find a pairs of seed graphs which are both not empty!
+                if len(self.seed_graphs[gi]['node_indices']) == 0:
+                    gi += 1
+                    gj += 1
                     continue
+
+                if len(self.seed_graphs[gj]['node_indices']) == 0:
+                    gj += 1
+                    continue
+
                 # We choose a random node from both subgraph seeded clusters of nodes and insert an edge
                 # there
-                i = random.choice([k for k, gi in enumerate(self.seed_graph_association)
-                                   if gi == graph_index and self.seed_graph_indices[k] < 0])
-                j = random.choice([k for k, gi in enumerate(self.seed_graph_association)
-                                   if gi == graph_index + 1])
+                i = random.choice([k for k, gk in enumerate(self.seed_graph_association)
+                                   if gk == gi and self.seed_graph_indices[k] < 0])
+                j = random.choice([k for k, gk in enumerate(self.seed_graph_association)
+                                   if gk == gj])
                 self.insert_edge(i, j)
                 additional_edge_count -= 1
 
+                gj += 1
+
         while additional_edge_count > 0:
             i, j = random.sample(self.node_indices_inserted, k=2)
 
